@@ -68,24 +68,26 @@ chebcoef(a,TRUE)
 f <- function(x) 1/(1+25*x^2)
 grid <- seq(-1,1,length.out=15)
 val <- f(grid)
-uc <- Vectorize(ucappx(val))
+uc <-ucappx(val)
 # and the Chebyshev
-ch <- Vectorize(chebappxf(f,15))
+ch <- chebappxf(f,15)
 # test it at 10 random points
-t(replicate(10,{a<-runif(1,-1,1); c(arg=a, uc=uc(a), true=f(a), cheb=ch(a))}))
+a <- runif(10)
+cbind(uc(a),ch(a),sapply(a,f))
 
-uc <- Vectorize(ucappxf(f,15,intervals=c(-1,1)))
+uc <- ucappxf(f,15,intervals=c(-1,1))
 a <- runif(1,-1,1)
 uc(a); ch(a); f(a)
 
 #polyharmonic splines
 f <- function(x) 10/(10+sum(sqrt(x)))
-knots <- matrix(runif(6000), 6)
+knots <- matrix(runif(6000,0,20), 6)
 phs <- polyh(f, knots, 3)
+round(fivenum(phs(knots)-apply(knots,2,f)),6)
 # test it in a random point
-a <- runif(6)
+a <- runif(6,0,20)
 f(a); phs(a)
-phs <- polyh(f,knots,5)
-phs(a)
-phs <- polyh(f,knots,-20)
-phs(a)
+phs <- polyh(f,knots,5,FALSE); nphs <- polyh(f,knots,5,TRUE)
+phs(a); nphs(a)
+phs <- polyh(f,knots,-1,FALSE); nphs <- polyh(f,knots,-1,TRUE)
+phs(a); nphs(a)
